import { indentLess, indentMore, insertTab } from '@codemirror/commands';
import { completionState, startCompletionEffect } from "./auto-complete/state.js";
import { startTooltip } from "./tooltip.js";

/**
 * Indent or insert a tab as appropriate.
 */
export var indentMoreOrInsertTab = function indentMoreOrInsertTab(view) {
  var from = view.state.selection.main.from;
  var to = view.state.selection.main.to;
  if (from !== to) {
    return indentMore(view);
  }
  var line = view.state.doc.lineAt(from);
  var before = view.state.doc.slice(line.from, from).toString();
  if (/^\s*$/.test(before)) {
    return indentMore(view);
  } else {
    return insertTab(view);
  }
};

/**
 * RegExp to test for leading whitespace
 */
var leadingWhitespaceRe = /^\s+$/;
export var indentOrCompletion = function indentOrCompletion(view) {
  var from = view.state.selection.main.from;
  var line = view.state.doc.lineAt(from);
  var shouldIndent = false;
  if (line.from === from) {
    shouldIndent = true;
  } else {
    shouldIndent = view.state.doc.slice(from - 1, from).toString().match(leadingWhitespaceRe) !== null;
  }
  if (shouldIndent) {
    return indentMoreOrInsertTab(view);
  } else {
    var cState = view.state.field(completionState, false);
    if (!cState) {
      return false;
    }
    view.dispatch({
      effects: startCompletionEffect.of(true)
    });
    return true;
  }
};
export var indentOrTooltip = function indentOrTooltip(view) {
  var from = view.state.selection.main.from;
  var line = view.state.doc.lineAt(from);
  var shouldIndentLess = view.state.doc.slice(line.from, from).toString().match(leadingWhitespaceRe) !== null;
  if (shouldIndentLess) {
    return indentLess(view);
  } else {
    return startTooltip(view);
  }
};